#include <SPI.h>
#include <MFRC522.h>

#define RST_PIN   9     // SPI Reset Pin
#define SS_PIN    10    // SPI Slave Select Pin

MFRC522 mfrc522(SS_PIN, RST_PIN);   // Instanz des MFRC522 erzeugen
#include <SoftwareSerial.h>
SoftwareSerial BTserial(2, 3); //integriert BT Kommunikation an Pin 2, 3

//definiert welcher RFID Chip zu welcher Zahl gehört
int wuerfel_1 = 122;
int wuerfel_3 = 95;
int wuerfel_2 = 130;
int wuerfel_4 = 114;
int wuerfel_5 = 106;
int wuerfel_6 = 118;

//Koordinaten Variablen
char aequator = 'a';
int location_a;
int location_b;

//Koordinates Goal
int goal = 0;
int goal_location_a = 0;
int goal_location_b = 0;

//BlueTooth Beacon that saves the wanted number
int beacon;

//Motor Variablen
int DIRL = 4;
int STEPL = 5;

int DIRR = 6;
int STEPR = 7;

int active = 0; //besagt anfänglich dass der roboter inaktiv ist

void setup() {
  // Diese Funktion wird einmalig beim Start ausgeführt
  Serial.begin(9600);  // Serielle Kommunikation mit dem PC initialisieren
  SPI.begin();         // Initialisiere SPI Kommunikation
  mfrc522.PCD_Init();  // Initialisiere MFRC522 Lesemodul
  BTserial.begin(9600); // Initialisiert die BT Kommunikation
  pinMode(DIRL, OUTPUT); //initiert die Motoren
  pinMode(STEPL, OUTPUT);

  pinMode(DIRR, OUTPUT);
  pinMode(STEPR, OUTPUT);

  digitalWrite(DIRL, LOW);
  digitalWrite(DIRR, LOW);
  digitalWrite(STEPL, LOW);
  digitalWrite(STEPR, LOW);
  //delay(25000); //notfall delay für wartung
}

void loop() {
  if (BTserial.available()) {
    beacon = BTserial.read();
    active = 1; // aktiviert den Roboter bei Bluetooth Eingabe

    if (beacon == 7) {
      goal = random(1, 7);

      switch (goal) { //simple Struktur zur Zuweisung der Koordinaten auf der Sphäre basierend auf der zufällig gewählten Nummer
        case 1: goal_location_a = 1;
          goal_location_b = 0;
          break;
        case 6: goal_location_a = 0;
          goal_location_b = 1;
          break;
        case 3: goal_location_a = 2;
          goal_location_b = 2;
          break;
        case 4: goal_location_a = 4;
          goal_location_b = 4;
          break;
        case 5: goal_location_a = 0;
          goal_location_b = 3;
          break;
        case 2: goal_location_a = 3;
          goal_location_b = 0;
          break;
      }
    }

    //Möglichkeit zum Eingeben konkreter Zielzahlen
    if (beacon >= 0 && beacon < 7) {
      goal = beacon;

      switch (goal) {
        case 1: goal_location_a = 1;
          goal_location_b = 0;
          break;
        case 6: goal_location_a = 0;
          goal_location_b = 1;
          break;
        case 3: goal_location_a = 2;
          goal_location_b = 2;
          break;
        case 4: goal_location_a = 4;
          goal_location_b = 4;
          break;
        case 5: goal_location_a = 0;
          goal_location_b = 3;
          break;
        case 2: goal_location_a = 3;
          goal_location_b = 0;
          break;
        case 0: goal_location_a = 0;
          goal_location_b = 0;
          break;
      }
    }
    Serial.println(beacon);
    Serial.println("Current Position");
    Serial.println(aequator);
    Serial.println(location_a);
    Serial.println(location_b);
    Serial.println("Goal Position");
    Serial.println(goal);
    Serial.println(goal_location_a);
    Serial.println(goal_location_b);

    //BT Communication List
    BTserial.println("Current Position");
    BTserial.println(aequator);
    BTserial.println(location_a);
    BTserial.println(location_b);
    BTserial.println("Goal Position");
    BTserial.println(goal);
    BTserial.println(goal_location_a);
    BTserial.println(goal_location_b);

  }

  while (active == 1 && goal != 0) { //limietiert Motorbewegung auf die Zeit wenn ein Ziel gegeben ist

    if (mfrc522.PICC_IsNewCardPresent() && mfrc522.PICC_ReadCardSerial() ) {
      int chipUid = 0;
      for (byte i = 1; i < 2; i++) {

        chipUid = mfrc522.uid.uidByte[i]; //saves the chipspecific UID in a variable
      }

      Serial.println(chipUid);


      if (chipUid == wuerfel_1) {
        location_a = 1;
        location_b = 0;
        Serial.println("1");
      }
      if (chipUid == wuerfel_6) {
        location_a = 0;
        location_b = 1;
        Serial.println("6");
      }
      if (chipUid == wuerfel_3) {
        location_a = 2;
        location_b = 2;
        Serial.println("3");
      }
      if (chipUid == wuerfel_4) {
        location_a = 4;
        location_b = 4;
        Serial.println("4");
      }
      if (chipUid == wuerfel_5) {
        location_a = 0;
        location_b = 3;
        Serial.println("5");
      }
      if (chipUid == wuerfel_2) {
        location_a = 3;
        location_b = 0;
        Serial.println("2");
      }

      Serial.println("Current Position");
      Serial.println(aequator);
      Serial.println(location_a);
      Serial.println(location_b);
      Serial.println("Goal Position");
      Serial.println(goal);
      Serial.println(goal_location_a);
      Serial.println(goal_location_b);

      //BT Communication List
      BTserial.println("Current Position");
      BTserial.println(aequator);
      BTserial.println(location_a);
      BTserial.println(location_b);
      BTserial.println("Goal Position");
      BTserial.println(goal);
      BTserial.println(goal_location_a);
      BTserial.println(goal_location_b);
      // Versetzt die gelesene Karte in einen Ruhemodus, um nach anderen Karten suchen zu können.
      mfrc522.PICC_HaltA();
      delay(1000);
    }

    if (aequator == 'a' && goal_location_a > 0 && location_a != goal_location_a ) {
      //Vorwärts Fahren Äquator a
      digitalWrite(STEPL, HIGH);
      digitalWrite(STEPR, HIGH);
      delay(25);
      digitalWrite(STEPL, LOW);
      digitalWrite(STEPR, LOW);
      delay(25);
    }


    if (aequator == 'b' && goal_location_b > 0 && location_b != goal_location_b) {
      //Vorwärts Fahren Äquator b
      digitalWrite(STEPL, HIGH);
      digitalWrite(STEPR, HIGH);
      delay(25);
      digitalWrite(STEPL, LOW);
      digitalWrite(STEPR, LOW);
      delay(25);
    }


    if (aequator == 'a' && goal_location_a == 0 && location_b == 0) {
      //zur Kreuzung fahren von Äquator a
      digitalWrite(STEPL, HIGH);
      digitalWrite(STEPR, HIGH);
      delay(25);
      digitalWrite(STEPL, LOW);
      digitalWrite(STEPR, LOW);
      delay(25);
    }


    if (aequator == 'b' && goal_location_b == 0 && location_a == 0) {
      //zur Kreuzung fahren von Äquator b
      digitalWrite(STEPL, HIGH);
      digitalWrite(STEPR, HIGH);
      delay(25);
      digitalWrite(STEPL, LOW);
      digitalWrite(STEPR, LOW);
      delay(25);
    }


    if (aequator == 'a' && goal_location_a == 0 && location_b > 0) {
      //drehen an Kreuzung nach b
      for (int i = 0; i <= 66; i++) {
        digitalWrite(STEPR, HIGH);
        digitalWrite(DIRL, HIGH);
        digitalWrite(STEPL, HIGH);
        delay(5);
        digitalWrite(STEPL, LOW);
        digitalWrite(STEPR, LOW);
        delay(5);
        //        Serial.println("turn");
      }
      BTserial.println("turn");
      aequator = 'b';
      digitalWrite(DIRL, LOW);
    }


    if (aequator == 'b' && goal_location_b == 0 && location_a > 0) {
      //drehen an Kreuzung nach a
      for (int i = 0; i <= 66; i++) {
        digitalWrite(STEPR, HIGH);
        digitalWrite(DIRL, HIGH);
        digitalWrite(STEPL, HIGH);
        delay(5);
        digitalWrite(STEPL, LOW);
        digitalWrite(STEPR, LOW);
        delay(5);
      }
      BTserial.println("turn");
      aequator = 'a';
      digitalWrite(DIRL, LOW);
    }
    if (BTserial.available()) {
      int emergencyBreak = BTserial.read(); //schreibt das Notsignal in eine Variable
      if (emergencyBreak = 8) {
        active = 0; // deaktiviert den Roboter bei Notsignal
      }
    }
    if (location_a == goal_location_a && location_b == goal_location_b) {
      active = 0; // deaktiviert den Roboter bei erreichtem Ziel
      BTserial.println("PARTYYYYYY");
    }
  }


}




//  while (location_a == goal_location_a && location_b == goal_location_b && BTserial.available() == 0) { //sorgt für ein stoppen wenn das ziel erreicht wurde + LED Blinken + Warten auf neuen BT Input
//    BTserial.println("PARTYYYYYY");
//    digitalWrite(LED_BUILTIN, HIGH);
//    delay(6500);                       // wait for a second
//    digitalWrite(LED_BUILTIN, LOW);
//  }
//  BTserial.println("ende while");
